@@ -138,6 +138,8 @@ typedef struct pkzip_data
 	int fileNum;
 	int allocatedChain;
 	INNER_FILE *chain;
+	
+	char *fileName;
 } PKZIP_DATA;
 
 
@@ -152,6 +154,7 @@ static int PKZip_LoadNextHeader(PKZIP_DATA *pkzip);
 static int PKZip_GetEndOfCentralDirectory(
 	PKZIP_DATA *pkzip,
 	EndOfCentralDirRecordBase *endOfCentralDir);
+static SDL_Archive *PKZip_Clone(SDL_Archive *orgArchive);
 
 static void Stored_SetMethod(SDL_Archive *archive);
 static long Stored_Size(SDL_Archive *archive);
@@ -224,6 +227,8 @@ SDL_Archive *Archive_FromPKZip(const char *file)
 	archive->filenum = PKZip_FileNumber;
 	archive->finish = PKZip_Finish;
 	
+	archive->clone = PKZip_Clone;
+	
 	archive->get_char = NULL;
 	archive->read = NULL;
 	archive->seek = NULL;
@@ -364,8 +369,16 @@ PKZIP_DATA *PKZip_Create(const char *file)
 		return NULL;
 	}
 	
+	pkzip->fileName = (char*)malloc(strlen(file) + 1);
+	if (pkzip->fileName == NULL) {
+		free(pkzip);
+		return NULL;
+	}
+	strcpy(pkzip->fileName, file);
+	
 	pkzip->rw = SDL_RWFromFile(file, "rb");
 	if (pkzip->rw == NULL) {
+		free(pkzip->fileName);
 		free(pkzip);
 		return NULL;
 	}
@@ -376,6 +389,8 @@ PKZIP_DATA *PKZip_Create(const char *file)
 		pkzip,
 		&endOfCentralDir);
 	if (ret == -1) {
+		free(pkzip->fileName);
+		SDL_RWclose(pkzip->rw);
 		free(pkzip);
 		return NULL;
 	}
@@ -389,6 +404,7 @@ PKZIP_DATA *PKZip_Create(const char *file)
 	
 	pkzip->chain = malloc(sizeof(INNER_FILE));
 	if (pkzip->chain == NULL) {
+		free(pkzip->fileName);
 		SDL_RWclose(pkzip->rw);
 		free(pkzip);
 		return NULL;
@@ -414,6 +430,67 @@ PKZIP_DATA *PKZip_Create(const char *file)
 }
 
 
+SDL_Archive *PKZip_Clone(SDL_Archive *orgArchive)
+{
+	PKZIP_DATA *pkzip;
+	PKZIP_DATA *orgPkzip;
+	SDL_Archive *archive;
+	
+	orgPkzip = (PKZIP_DATA*)orgArchive->data;
+	
+	pkzip = malloc(sizeof(PKZIP_DATA));
+	if (pkzip == NULL) {
+		return NULL;
+	}
+	
+	pkzip->fileName = (char*)malloc(strlen(orgPkzip->fileName) + 1);
+	if (pkzip->fileName == NULL) {
+		free(pkzip);
+		return NULL;
+	}
+	strcpy(pkzip->fileName, orgPkzip->fileName);
+	
+	pkzip->chain = (INNER_FILE*)malloc(
+		sizeof(INNER_FILE) * orgPkzip->fileNum);
+	if (pkzip->chain == NULL) {
+		free(pkzip->fileName);
+		free(pkzip);
+	}
+	memcpy(pkzip->chain, orgPkzip->chain,
+		sizeof(INNER_FILE) * orgPkzip->fileNum);
+	
+	pkzip->openNumber = -1;
+	
+	archive = Archive_Alloc();
+	if (archive == NULL) {
+		free(pkzip->fileName);
+		free(pkzip->chain);
+		free(pkzip);
+		return NULL;
+	}
+	
+	archive->data = pkzip;
+	
+	archive->open = PKZip_Open;
+	archive->numopen = PKZip_NumOpen;
+	archive->name2index = PKZip_NameToIndex;
+	archive->filenum = PKZip_FileNumber;
+	archive->finish = PKZip_Finish;
+	
+	archive->clone = PKZip_Clone;
+	
+	archive->get_char = NULL;
+	archive->read = NULL;
+	archive->seek = NULL;
+	archive->size = NULL;
+	archive->tell = NULL;
+	archive->eof = NULL;
+	archive->close = NULL;
+	
+	return archive;
+}
+
+
 int PKZip_Open(SDL_Archive *archive, const char *file)
 {
 	return PKZip_NumOpen(archive, PKZip_NameToIndex(archive, file));
